#
# $Id: process_transport_syslog_line.pl,v 1.1 2001/12/21 08:33:44 fpmip Exp $
#

# Author : Tycho Fruru <tycho@fruru.com>

# Copyright (C) 2001 Tycho Fruru <tycho@fruru.com>
# Copyright (C) 2001 Conostix S.A. Luxembourg - Tycho Fruru <tycho@conostix.com>

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#

print "Registering process_transport_syslog_line\n";

sub process_transport_syslog_line($$$$$) {
	my $dbh = shift;
	my $entry = shift;
	my $function_name = shift;
	my $table_name = shift;
	my $r_sql_hash = shift;
	
	my $current_entry;
	my $index = 0;
	my $rc = 1;
	
	while (($rc == 1) and (defined($entry->{'syslog-line'}->[$index]))) {
		my %sql_hash = %{$r_sql_hash};
		my $decoded_line = decode_entry($entry->{'syslog-line'}->[$index]);
		
		my $log_message;
		
		$sql_hash{'message'}=$decoded_line;
		
		if ($decoded_line =~ /^\s*(\w+\s+\d+\s+\d+:\d+:\d+)\s+(\w+)\s+(([^:]+)\:)?\s*(\w*.*)$/) {
	
			$sql_hash{'syslog_date'} = $1;
			$sql_hash{'host'} = $2;
			$sql_hash{'logging_program'} = $4;
			$sql_hash{'pid'} = 0;
			$log_message = $5;
		}
			
		if (!defined($sql_hash{'logging_program'})) {
			delete $sql_hash{'logging_program'};
		}
	
		if ($sql_hash{'logging_program'} =~ /^(.+)\[(\d+)\]$/) {
			$sql_hash{'logging_program'} = $1;
			$sql_hash{'pid'} = $2;
		}
		
		if ($sql_hash{'syslog_date'} =~ /^(\w+)\s+(\d+)\s+(\d+:\d+:\d+)$/) {
			my $syslog_month = lc($1);
			my $syslog_day = $2;
			my $syslog_time = $3;
		
		# heuristic to get the right year (which is not included in syslog lines !)
		# try the current year, one year ahead and one year behind
		# the date which is the nearest from the current one is supposed to be the right one
		
			my ($sec,$min,$hour,$mday,$current_mon,$current_year,$wday,$yday) =
                        	                                   gmtime(time);
							   
			$current_year+=1900; # use the correct 4 digit version
		
			$syslog_month = $process_transport_syslog_line::months{$syslog_month};
			if (!$syslog_month) { $syslog_month = 0 };
		
			my $current_seconds = time;
			my $syslog_current_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year);
			my $syslog_next_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year+1);
			my $syslog_previous_year = timegm(0,0,0,$syslog_day,$syslog_month,$current_year-1);
		
			my $difference_current = abs($current_seconds - $syslog_current_year);
		
			my $difference_next = abs($current_seconds - $syslog_next_year);
			my $difference_previous = abs($current_seconds - $syslog_previous_year);
		
			my $guessed_year;
		
			if (($difference_current < $difference_next) &&
			    ($difference_current < $difference_previous)) {
				$guessed_year = $current_year;
			} elsif ($difference_previous < $difference_next) {
				$guessed_year = $current_year-1;
			} else {
				$guessed_year = $current_year+1;
			}
		
			$syslog_month+=1; # to get back to human months
			$sql_hash{'time'}="$guessed_year-$syslog_month-$syslog_day $syslog_time";
			
		}

		
		$rc = &$function_name($dbh,$log_message,$table_name,\%sql_hash);
		$index++;
	}
	
	show_msg(5,"return value for process_transport_syslog_line : $rc\n");
	return $rc;
}

%process_transport_syslog_line::months=
	('jan'=>0,'feb'=>1,'mar'=>2,'apr'=>3,'may'=>4,'jun'=>5,
	 'jul'=>6,'aug'=>7,'sep'=>8,'oct'=>9,'nov'=>10,'dec'=>11);

return 1;
